# 堆分配器

为了使用 `Rust` 的 `alloc`，因此在内核中需要实现一个针对 `Rust` 语言的分配器，且实现 `GlobalAlloc` 接口，这里我们采用已经比较完善的 `buddy_system_allocator`。

## 高半核的好处

在我们已经实现了 `higher-half kernel` 的基础上，我们能够在整个程序的尾部（虚拟内存）添加一个堆分配器，即内核堆，在堆分配器内存不够的时候会在内核中出发页表缺失异常，然后在异常处理函数中进行页表的分配和映射，然后返回相应的位置即可继续运行，栈同理。


```rust
#![no_std]

use buddy_system_allocator::LockedHeap;

// 堆大小
const HEAP_SIZE: usize = 0x0008_0000;

// 堆空间
static mut HEAP: [u8; HEAP_SIZE] = [0; HEAP_SIZE];

// 堆内存分配器
#[global_allocator]
static HEAP_ALLOCATOR: LockedHeap<64> = LockedHeap::empty();

// 初始化堆内存分配器
pub fn init() {
    unsafe {
        HEAP_ALLOCATOR
            .lock()
            .init(HEAP.as_ptr() as usize, HEAP_SIZE);
    }
}
```